import gleam/dynamic/decode
import gleam/option.{type Option}
import spacetraders_sdk/models/agent.{type Agent}
import spacetraders_sdk/models/agent_event.{type AgentEvent}
import spacetraders_sdk/models/agent_symbol.{type AgentSymbol}
import spacetraders_sdk/models/public_agent.{type PublicAgent}
import spacetraders_sdk/utils/api.{type ApiResponse, type PagedData}
import spacetraders_sdk/utils/auth.{type AgentToken, AgentAuth, NoAuth}

pub fn list_public_agents(
  page: Option(Int),
  limit: Option(Int),
) -> ApiResponse(PagedData(List(PublicAgent))) {
  let request = api.get_page(NoAuth, "/agents", page, limit)
  use response <- api.try_send(request)
  case response.status {
    200 ->
      api.parse_paged_data_response(
        response,
        decode.list(public_agent.decoder()),
      )
    _ -> api.parse_error_response(response)
  }
}

pub fn get_public_agent(agent_symbol: AgentSymbol) -> ApiResponse(PublicAgent) {
  let request =
    api.get(NoAuth, "/agents/" <> agent_symbol.to_string(agent_symbol))
  use response <- api.try_send(request)
  case response.status {
    200 -> api.parse_data_response(response, public_agent.decoder())
    _ -> api.parse_error_response(response)
  }
}

pub fn get_agent(token: AgentToken) -> ApiResponse(Agent) {
  let request = api.get(AgentAuth(token), "/my/agent")
  use response <- api.try_send(request)
  case response.status {
    200 -> api.parse_data_response(response, agent.decoder())
    _ -> api.parse_error_response(response)
  }
}

pub fn get_agent_events(token: AgentToken) -> ApiResponse(List(AgentEvent)) {
  let request = api.get(AgentAuth(token), "/my/agent/events")
  use response <- api.try_send(request)
  case response.status {
    200 -> api.parse_data_response(response, decode.list(agent_event.decoder()))
    _ -> api.parse_error_response(response)
  }
}
