import gleam/dict.{type Dict}
import gleam/dynamic/decode
import spacetraders_sdk/models/trade_symbol.{type TradeSymbol}
import spacetraders_sdk/utils/api.{type ApiResponse}
import spacetraders_sdk/utils/auth.{NoAuth}

pub type ExportToImportMap =
  Dict(TradeSymbol, List(TradeSymbol))

pub fn get_supply_chain() -> ApiResponse(ExportToImportMap) {
  let request = api.get(NoAuth, "/market/supply-chain")
  use response <- api.try_send(request)
  case response.status {
    200 ->
      api.parse_data_response(
        response,
        decode.field(
          "exportToImportMap",
          decode.dict(
            trade_symbol.decoder(),
            decode.list(trade_symbol.decoder()),
          ),
          decode.success,
        ),
      )
    _ -> api.parse_error_response(response)
  }
}
