import gleam/dynamic/decode
import gleam/option.{type Option}
import spacetraders_sdk/models/faction.{type Faction}
import spacetraders_sdk/models/faction_symbol.{type FactionSymbol}
import spacetraders_sdk/utils/api.{type ApiResponse, type PagedData}
import spacetraders_sdk/utils/auth.{type AgentToken, AgentAuth, NoAuth}

pub fn list_factions(
  page: Option(Int),
  limit: Option(Int),
) -> ApiResponse(PagedData(List(Faction))) {
  let request = api.get_page(NoAuth, "/factions", page, limit)
  use response <- api.try_send(request)
  case response.status {
    200 ->
      api.parse_paged_data_response(response, decode.list(faction.decoder()))
    _ -> api.parse_error_response(response)
  }
}

pub fn get_faction(symbol: FactionSymbol) -> ApiResponse(Faction) {
  let request =
    api.get(NoAuth, "/factions/" <> faction_symbol.to_string(symbol))
  use response <- api.try_send(request)
  case response.status {
    200 -> api.parse_data_response(response, faction.decoder())
    _ -> api.parse_error_response(response)
  }
}

pub type FactionReputation {
  FactionReputation(symbol: FactionSymbol, reputation: Int)
}

pub fn get_my_factions(
  token: AgentToken,
  page: Option(Int),
  limit: Option(Int),
) -> ApiResponse(PagedData(List(FactionReputation))) {
  let request = api.get_page(AgentAuth(token), "/my/factions", page, limit)
  use response <- api.try_send(request)
  case response.status {
    200 ->
      api.parse_paged_data_response(
        response,
        decode.list({
          use symbol <- decode.field("symbol", faction_symbol.decoder())
          use reputation <- decode.field("reputation", decode.int)
          decode.success(FactionReputation(symbol:, reputation:))
        }),
      )
    _ -> api.parse_error_response(response)
  }
}
